////////////////////////////////////////////////////////////////////////////////
// File         : HnetPrint.cpp
// Subsystem    : Graph
// Component    : H-net
// Author       : Josef Huebl, (Josef.Huebl@sss.de)
// Description  : implementation of the class HnetPrint
////////////////////////////////////////////////////////////////////////////////
// Copyright (C) Triple-S GmbH (www.sss.de) 2003
// Alle Rechte vorbehalten. All Rights Reserved.
//
// The reproduction, transmission or use of this document or its contents is
// not permitted without express written authority.
// Offenders will be liable for damages. All rights, including rights created
// by patent grant or registration of a utility model or design, are reserved.
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// History:
// 19.09.03    Josef Huebl       created
// 17.12.03    Josef Huebl       1. released version.
// 30.05.06    Josef Huebl       some extensions
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Hints:      
//     
//     
////////////////////////////////////////////////////////////////////////////////


#include "HnetPrint.h"
#include "HnetUniversePrint.h"

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////


HnetPrint::HnetPrint( HnetUniversePrint *universe, const char *name ) 
	: ObjPrint( "H-Net", name ), Hnet( universe )
{
	SetNodeID( Root(), 0 );
}


HnetPrint::HnetPrint( HnetPrint &hnet, const char *name ) 
	: ObjPrint( hnet, name ), Hnet( hnet.m_universe ) 
{
	Copy( hnet );
};

HnetPrint::HnetPrint(  HnetUniversePrint *universe, Hnet &hnet, const char *name ) 
	: ObjPrint( "H-Net", name ), Hnet( hnet), m_universePrint( universe ) 
{
	RenumberNewSubcentre();
};

HnetPrint::~HnetPrint()
{

}


//////////////////////////////////////////////////////////////////////
// public
//////////////////////////////////////////////////////////////////////


HnetPrint& HnetPrint::operator=( HnetPrint &hnet)
{
	if (this != &hnet)
		Copy( hnet );

	return *this;
}



void HnetPrint::PrintBody( void ) const
{
	Node s;

	PrintSubcenter( Root() );
	printf( "\n" );

	s = FirstSubcentre(); 

	while ( s )
	{
		if ( s != Root() )
			PrintSubcenter( s );
		s = NextSubcentre( s ) ;
	}

	printf( "\n  connections:\n" );

	PrintConnections( Root() );
	s = FirstSubcentre(); 

	while ( s )
	{
		if ( s != Root())
			PrintConnections( s );
		s = NextSubcentre( s ) ;
	}
}


char *HnetPrint::Str( const Node &n) const
{
	static char str[ sizeof(NodeID)*10+2 ];

	NodeID id = GetNodeID( n );
	Node s;

	NodeType nodeType = GetNodeType( n );

	switch ( nodeType )
	{
	case NODE_UNKNOWN: // should not happen
		sprintf( str, "?%02u", id );
		break;
	case NODE_SUBCENTRE:
		sprintf( str, "s%02u", id );
		break;
	case NODE_ENTRY:
		s = n.FirstOutgoingEdge().TargetNode();
		if ( s == Root())
			sprintf( str, "A%02u", id ); //outer exit
		else
			sprintf( str, "e%02u", id ); //inner entry
		break;
	case NODE_EXIT:
		s = n.FirstIngoingEdge().SourceNode();
		if ( s == Root())
			sprintf( str, "E%02u", id ); //outer entry
		else
			sprintf( str, "a%02u", id ); //inner exit
		break;
	}

	return str;
}

//////////////////////////////////////////////////////////////////////
// private
//////////////////////////////////////////////////////////////////////

void HnetPrint::PrintSubcenter( const Node &s ) const
{
	Node n;
	Edge e;
	IsoTypeNode t = m_isoType[ s ];

	printf( "  t%02u  %s ( ", m_universePrint->GetIsoTypeID( t ), Str( s ) );

	n = FirstEntry( e, s );
	while ( n )
	{
		printf( "%s ", Str( n ) );
		n = NextEntry( e );
	}

	printf( "/  " );

	n = FirstExit( e, s );
	while ( n )
	{
		printf( "%s ", Str( n ) );
		n = NextExit( e );
	}

	printf( ")\n" );
}


void HnetPrint::PrintConnections( const Node &s ) const
{
	Node a, n, sn;
	Edge e, en;
	bool first;
	char str[ sizeof(NodeID)*10+2 ];


	a = FirstExit( e, s );
	while ( a )
	{
		first = true;
		en = a.FirstOutgoingEdge();
		while ( en )
		{
			n = en.TargetNode();
			if (first )
			{
				first = false;
				strcpy( str, Str( s ) );
				printf( "  (%s, %s) -> ",  str, Str( a ) );
			}

			sn = n.FirstOutgoingEdge().TargetNode();
			strcpy( str, Str( sn ) );
			printf( "(%s, %s) ",  str, Str( n ) );

			en = en.NextOutgoingEdge();
		}

		if (!first)
			printf( "\n" );

		a = NextExit( e );
	}
}


void HnetPrint::Copy( HnetPrint &hnet)
{
	Node n, nOrig;

	ObjPrint::Copy( hnet ); 
	Hnet::Copy( hnet );
	NodesNumbers::Copy( hnet, Root(), hnet.Root() );
}

void HnetPrint::RenumberYX( const Node &s )
{
	int i;
	Edge e;
	Node nn = FirstEntry( e, s );

	for ( i = 0; i < NumberOfEntries( s ); i++ )
	{
		SetNodeID( nn, i );
		nn = NextEntry( e );
	}

	nn = FirstExit( e, s );

	for ( i = 0; i < NumberOfExits( s ); i++ )
	{
		SetNodeID( nn, i );
		nn = NextExit( e );
	}
}

void HnetPrint::RenumberSubcentre( const Node &s )
{
	RenumberYX( s );

	if ( s == Root() )
	{
		SetNodeID( s, 0 );
		return;
	}

	Node ss = LastSubcentre();

	while ( ss && (GetNodeID( ss ) == InvalidNodeID ) )
	{
		ss = PrevSubcentre( ss );
	}

	if ( ss )
		SetNodeID( s, GetNodeID( ss ) + 1 );
	else
		SetNodeID( s, GetNodeID( Root() ) + 1 );
}


void HnetPrint::RenumberNewSubcentre( void )
{

	if ( GetNodeID( Root() ) == InvalidNodeID )
	{
		RenumberSubcentre( Root() );
	}

	Node s = FirstSubcentre();
	while ( s ) 
	{
		if ( GetNodeID( s ) == InvalidNodeID )
		{
			RenumberSubcentre( s );
		}

		s = NextSubcentre( s );
	}
}
