////////////////////////////////////////////////////////////////////////////////
// File         : SnetPrint.cpp
// Subsystem    : Graph
// Component    : S-net
// Author       : Josef Huebl, (Josef.Huebl@sss.de)
// Description  : implementation of the class SnetPrint
////////////////////////////////////////////////////////////////////////////////
// Copyright (C) Triple-S GmbH (www.sss.de) 2003
// Alle Rechte vorbehalten. All Rights Reserved.
//
// The reproduction, transmission or use of this document or its contents is
// not permitted without express written authority.
// Offenders will be liable for damages. All rights, including rights created
// by patent grant or registration of a utility model or design, are reserved.
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// History:
// 11.09.03    Josef Huebl       created
// 17.12.03    Josef Huebl       1. released version.
// 30.05.06    Josef Huebl       some extensions
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Hints:      
//     
//     
////////////////////////////////////////////////////////////////////////////////

#include "SnetPrint.h"

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

SnetPrint::SnetPrint( const char *name ) : ObjPrint( "S-Net", name )
{
	NewNodeID( m_sx );
}


SnetPrint::SnetPrint( SnetPrint &snet, const char *name ) : ObjPrint( snet, name ) 
{
	Copy( snet );
};


SnetPrint::SnetPrint( Snet &snet, const char *name ) : ObjPrint( name ) 
{
	Snet::Copy( snet );
	Renumber();
};


SnetPrint::~SnetPrint()
{

}


//////////////////////////////////////////////////////////////////////
// public
//////////////////////////////////////////////////////////////////////


SnetPrint& SnetPrint::operator=( SnetPrint &snet)
{
	if (this != &snet)
		Copy( snet );

	return *this;
}


Node SnetPrint::AddOuterEntry( void )
{
	Node n = Snet::AddOuterEntry();

	if (n)
		NewNodeID( n );

	return n;
}


Node SnetPrint::AddOuterExit( void )
{
	Node n = Snet::AddOuterExit();

	if (n)
		NewNodeID( n );

	return n;
}


Node SnetPrint::AddSubcentre( int numEntries, int numExits )
{
	Node s, n;
	Edge e;

	s = Snet::AddSubcentre( numEntries, numExits );

	if ( s )
	{
		NewNodeID( s );

		n = FirstEntry( e, s );
		while ( n )
		{
			NewNodeID( n );
			n = NextEntry( e );
		}

		n = FirstExit( e, s );
		while ( n )
		{
			NewNodeID( n );
			n = NextExit( e );
		}
	}

	return s;
}


Node SnetPrint::Replace( Node &s, SnetPrint &snet ) 
{
	Node n, FirstNode = Snet::Replace( s, snet );
	n = FirstNode;

	while ( n )
	{
		NewNodeID( n );
		n = n.NextNode();
	}

	return FirstNode;
}


void SnetPrint::TransComplement( SnetPrint &complement, NodeAttribute< Node > &originNode )
{
	Snet::TransComplement( complement, originNode );

	Node n = complement.Root();

	while ( n  )  
	{
		NodeID id;

		if ( originNode[ n ] )
		{
			id = GetNodeID( originNode[ n ] );
			complement.SetNodeID( n , id );
		}

		n = n.NextNode();
	}

	complement.RenumberNew( );
}


void SnetPrint::PrintBody( void ) const
{
	Node s;

	PrintSubcenter( m_sx );
	printf( "\n" );

	s = FirstSubcentre(); 

	while ( s )
	{
		if ( s != m_sx)
			PrintSubcenter( s );
		s = NextSubcentre( s ) ;
	}

	printf( "\n  connections:\n" );

	PrintConnections( m_sx );
	s = FirstSubcentre(); 

	while ( s )
	{
		if ( s != m_sx)
			PrintConnections( s );
		s = NextSubcentre( s ) ;
	}
}


char *SnetPrint::Str( const Node &n) const
{
	static char str[ sizeof(NodeID)*10+2 ];

	NodeID id = GetNodeID( n );
	Node s;

	switch ( m_nodeType[ n ] )
	{
	case NODE_UNKNOWN: // should not happen
		sprintf( str, "?%02u", id );
		break;
	case NODE_SUBCENTRE:
		sprintf( str, "s%02u", id );
		break;
	case NODE_ENTRY:
		s = n.FirstOutgoingEdge().TargetNode();
		if ( s == m_sx)
			sprintf( str, "A%02u", id ); //outer exit
		else
			sprintf( str, "e%02u", id ); //inner entry
		break;
	case NODE_EXIT:
		s = n.FirstIngoingEdge().SourceNode();
		if ( s == m_sx)
			sprintf( str, "E%02u", id ); //outer entry
		else
			sprintf( str, "a%02u", id ); //inner exit
		break;
	}

	return str;
}


void SnetPrint::PrintRelation( const NodeAttribute< Relation > &relation) const
{
	Node s = FirstSubcentre();
	printf( "Relations of %s\n{\n", GetFullName() );

	char name[128];

	while ( s )
	{
		if ( relation[ s ].Size() != 0 )
		{
			sprintf( name, "of subcentre %d", GetNodeID( s ) );
			RelationPrint rp( *this, relation[ s ], name );
			rp.Print();
		}
		s = NextSubcentre( s );
	}
	printf( "} \n" );
}

//////////////////////////////////////////////////////////////////////
// private
//////////////////////////////////////////////////////////////////////

void SnetPrint::PrintSubcenter( const Node &s ) const
{
	Node n;
	Edge e;

	printf( "  %s ( ", Str( s ) );

	n = FirstEntry( e, s );
	while ( n )
	{
		printf( "%s ", Str( n ) );
		n = NextEntry( e );
	}

	printf( "/  " );

	n = FirstExit( e, s );
	while ( n )
	{
		printf( "%s ", Str( n ) );
		n = NextExit( e );
	}

	printf( ")\n" );
}


void SnetPrint::PrintConnections( const Node &s ) const
{
	Node a, n, sn;
	Edge e, en;
	bool first;
	char str[ sizeof(NodeID)*10+2 ];


	a = FirstExit( e, s );
	while ( a )
	{
		first = true;
		en = a.FirstOutgoingEdge();
		while ( en )
		{
			n = en.TargetNode();
			if (first )
			{
				first = false;
				strcpy( str, Str( s ) );
				printf( "  (%s, %s) -> ",  str, Str( a ) );
			}

			sn = n.FirstOutgoingEdge().TargetNode();
			strcpy( str, Str( sn ) );
			printf( "(%s, %s) ",  str, Str( n ) );

			en = en.NextOutgoingEdge();
		}

		if (!first)
			printf( "\n" );

		a = NextExit( e );
	}
}


void SnetPrint::Copy( SnetPrint &snet)
{
	Node n, nOrig;

	ObjPrint::Copy( snet ); 
	Snet::Copy( snet );
	NodesNumbers::Copy( snet, m_g.FirstNode(), snet.m_g.FirstNode() );
}



